from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="SSHBinding")


@attr.s(auto_attribs=True)
class SSHBinding:
    """  """

    address: Union[Unset, str] = UNSET
    port: Union[Unset, int] = UNSET
    apply_proxy_config: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        address = self.address
        port = self.port
        apply_proxy_config = self.apply_proxy_config

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if address is not UNSET:
            field_dict["address"] = address
        if port is not UNSET:
            field_dict["port"] = port
        if apply_proxy_config is not UNSET:
            field_dict["apply_proxy_config"] = apply_proxy_config

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        address = d.pop("address", UNSET)

        port = d.pop("port", UNSET)

        apply_proxy_config = d.pop("apply_proxy_config", UNSET)

        ssh_binding = cls(
            address=address,
            port=port,
            apply_proxy_config=apply_proxy_config,
        )

        ssh_binding.additional_properties = d
        return ssh_binding

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="VersionInfo")


@attr.s(auto_attribs=True)
class VersionInfo:
    """  """

    version: Union[Unset, str] = UNSET
    build_date: Union[Unset, str] = UNSET
    commit_hash: Union[Unset, str] = UNSET
    features: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        version = self.version
        build_date = self.build_date
        commit_hash = self.commit_hash
        features: Union[Unset, List[str]] = UNSET
        if not isinstance(self.features, Unset):
            features = self.features

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if version is not UNSET:
            field_dict["version"] = version
        if build_date is not UNSET:
            field_dict["build_date"] = build_date
        if commit_hash is not UNSET:
            field_dict["commit_hash"] = commit_hash
        if features is not UNSET:
            field_dict["features"] = features

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        version = d.pop("version", UNSET)

        build_date = d.pop("build_date", UNSET)

        commit_hash = d.pop("commit_hash", UNSET)

        features = cast(List[str], d.pop("features", UNSET))

        version_info = cls(
            version=version,
            build_date=build_date,
            commit_hash=commit_hash,
            features=features,
        )

        version_info.additional_properties = d
        return version_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AdminPermissions(str, Enum):
    VALUE_0 = "*"
    ADD_USERS = "add_users"
    EDIT_USERS = "edit_users"
    DEL_USERS = "del_users"
    VIEW_USERS = "view_users"
    VIEW_CONNS = "view_conns"
    CLOSE_CONNS = "close_conns"
    VIEW_STATUS = "view_status"
    MANAGE_ADMINS = "manage_admins"
    QUOTA_SCANS = "quota_scans"
    MANAGE_SYSTEM = "manage_system"
    MANAGE_DEFENDER = "manage_defender"
    VIEW_DEFENDER = "view_defender"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PatternsFilter")


@attr.s(auto_attribs=True)
class PatternsFilter:
    """  """

    path: Union[Unset, str] = UNSET
    allowed_patterns: Union[Unset, List[str]] = UNSET
    denied_patterns: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        path = self.path
        allowed_patterns: Union[Unset, List[str]] = UNSET
        if not isinstance(self.allowed_patterns, Unset):
            allowed_patterns = self.allowed_patterns

        denied_patterns: Union[Unset, List[str]] = UNSET
        if not isinstance(self.denied_patterns, Unset):
            denied_patterns = self.denied_patterns

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if path is not UNSET:
            field_dict["path"] = path
        if allowed_patterns is not UNSET:
            field_dict["allowed_patterns"] = allowed_patterns
        if denied_patterns is not UNSET:
            field_dict["denied_patterns"] = denied_patterns

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        path = d.pop("path", UNSET)

        allowed_patterns = cast(List[str], d.pop("allowed_patterns", UNSET))

        denied_patterns = cast(List[str], d.pop("denied_patterns", UNSET))

        patterns_filter = cls(
            path=path,
            allowed_patterns=allowed_patterns,
            denied_patterns=denied_patterns,
        )

        patterns_filter.additional_properties = d
        return patterns_filter

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.data_provider_status import DataProviderStatus
from ..models.ftp_service_status import FTPServiceStatus
from ..models.services_status_defender import ServicesStatusDefender
from ..models.ssh_service_status import SSHServiceStatus
from ..models.web_dav_service_status import WebDAVServiceStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="ServicesStatus")


@attr.s(auto_attribs=True)
class ServicesStatus:
    """  """

    ssh: Union[Unset, SSHServiceStatus] = UNSET
    ftp: Union[Unset, FTPServiceStatus] = UNSET
    webdav: Union[Unset, WebDAVServiceStatus] = UNSET
    data_provider: Union[Unset, DataProviderStatus] = UNSET
    defender: Union[Unset, ServicesStatusDefender] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        ssh: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.ssh, Unset):
            ssh = self.ssh.to_dict()

        ftp: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.ftp, Unset):
            ftp = self.ftp.to_dict()

        webdav: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.webdav, Unset):
            webdav = self.webdav.to_dict()

        data_provider: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.data_provider, Unset):
            data_provider = self.data_provider.to_dict()

        defender: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.defender, Unset):
            defender = self.defender.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if ssh is not UNSET:
            field_dict["ssh"] = ssh
        if ftp is not UNSET:
            field_dict["ftp"] = ftp
        if webdav is not UNSET:
            field_dict["webdav"] = webdav
        if data_provider is not UNSET:
            field_dict["data_provider"] = data_provider
        if defender is not UNSET:
            field_dict["defender"] = defender

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        ssh: Union[Unset, SSHServiceStatus] = UNSET
        _ssh = d.pop("ssh", UNSET)
        if not isinstance(_ssh, Unset):
            ssh = SSHServiceStatus.from_dict(_ssh)

        ftp: Union[Unset, FTPServiceStatus] = UNSET
        _ftp = d.pop("ftp", UNSET)
        if not isinstance(_ftp, Unset):
            ftp = FTPServiceStatus.from_dict(_ftp)

        webdav: Union[Unset, WebDAVServiceStatus] = UNSET
        _webdav = d.pop("webdav", UNSET)
        if not isinstance(_webdav, Unset):
            webdav = WebDAVServiceStatus.from_dict(_webdav)

        data_provider: Union[Unset, DataProviderStatus] = UNSET
        _data_provider = d.pop("data_provider", UNSET)
        if not isinstance(_data_provider, Unset):
            data_provider = DataProviderStatus.from_dict(_data_provider)

        defender: Union[Unset, ServicesStatusDefender] = UNSET
        _defender = d.pop("defender", UNSET)
        if not isinstance(_defender, Unset):
            defender = ServicesStatusDefender.from_dict(_defender)

        services_status = cls(
            ssh=ssh,
            ftp=ftp,
            webdav=webdav,
            data_provider=data_provider,
            defender=defender,
        )

        services_status.additional_properties = d
        return services_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.connection_status_protocol import ConnectionStatusProtocol
from ..models.transfer import Transfer
from ..types import UNSET, Unset

T = TypeVar("T", bound="ConnectionStatus")


@attr.s(auto_attribs=True)
class ConnectionStatus:
    """  """

    username: Union[Unset, str] = UNSET
    connection_id: Union[Unset, str] = UNSET
    client_version: Union[Unset, str] = UNSET
    remote_address: Union[Unset, str] = UNSET
    connection_time: Union[Unset, int] = UNSET
    command: Union[Unset, str] = UNSET
    last_activity: Union[Unset, int] = UNSET
    protocol: Union[Unset, ConnectionStatusProtocol] = UNSET
    active_transfers: Union[Unset, List[Transfer]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        username = self.username
        connection_id = self.connection_id
        client_version = self.client_version
        remote_address = self.remote_address
        connection_time = self.connection_time
        command = self.command
        last_activity = self.last_activity
        protocol: Union[Unset, str] = UNSET
        if not isinstance(self.protocol, Unset):
            protocol = self.protocol.value

        active_transfers: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.active_transfers, Unset):
            active_transfers = []
            for active_transfers_item_data in self.active_transfers:
                active_transfers_item = active_transfers_item_data.to_dict()

                active_transfers.append(active_transfers_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if username is not UNSET:
            field_dict["username"] = username
        if connection_id is not UNSET:
            field_dict["connection_id"] = connection_id
        if client_version is not UNSET:
            field_dict["client_version"] = client_version
        if remote_address is not UNSET:
            field_dict["remote_address"] = remote_address
        if connection_time is not UNSET:
            field_dict["connection_time"] = connection_time
        if command is not UNSET:
            field_dict["command"] = command
        if last_activity is not UNSET:
            field_dict["last_activity"] = last_activity
        if protocol is not UNSET:
            field_dict["protocol"] = protocol
        if active_transfers is not UNSET:
            field_dict["active_transfers"] = active_transfers

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        username = d.pop("username", UNSET)

        connection_id = d.pop("connection_id", UNSET)

        client_version = d.pop("client_version", UNSET)

        remote_address = d.pop("remote_address", UNSET)

        connection_time = d.pop("connection_time", UNSET)

        command = d.pop("command", UNSET)

        last_activity = d.pop("last_activity", UNSET)

        protocol: Union[Unset, ConnectionStatusProtocol] = UNSET
        _protocol = d.pop("protocol", UNSET)
        if not isinstance(_protocol, Unset):
            protocol = ConnectionStatusProtocol(_protocol)

        active_transfers = []
        _active_transfers = d.pop("active_transfers", UNSET)
        for active_transfers_item_data in _active_transfers or []:
            active_transfers_item = Transfer.from_dict(active_transfers_item_data)

            active_transfers.append(active_transfers_item)

        connection_status = cls(
            username=username,
            connection_id=connection_id,
            client_version=client_version,
            remote_address=remote_address,
            connection_time=connection_time,
            command=command,
            last_activity=last_activity,
            protocol=protocol,
            active_transfers=active_transfers,
        )

        connection_status.additional_properties = d
        return connection_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="BanStatus")


@attr.s(auto_attribs=True)
class BanStatus:
    """  """

    date_time: Union[Unset, None, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        date_time: Union[Unset, None, str] = UNSET
        if not isinstance(self.date_time, Unset):
            date_time = self.date_time.isoformat() if self.date_time else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if date_time is not UNSET:
            field_dict["date_time"] = date_time

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        date_time = None
        _date_time = d.pop("date_time", UNSET)
        if _date_time is not None and not isinstance(_date_time, Unset):
            date_time = isoparse(_date_time)

        ban_status = cls(
            date_time=date_time,
        )

        ban_status.additional_properties = d
        return ban_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.secret import Secret
from ..types import UNSET, Unset

T = TypeVar("T", bound="SFTPFsConfig")


@attr.s(auto_attribs=True)
class SFTPFsConfig:
    """  """

    endpoint: Union[Unset, str] = UNSET
    username: Union[Unset, str] = UNSET
    password: Union[Unset, Secret] = UNSET
    private_key: Union[Unset, Secret] = UNSET
    fingerprints: Union[Unset, List[str]] = UNSET
    prefix: Union[Unset, str] = UNSET
    disable_concurrent_reads: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        endpoint = self.endpoint
        username = self.username
        password: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.password, Unset):
            password = self.password.to_dict()

        private_key: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.private_key, Unset):
            private_key = self.private_key.to_dict()

        fingerprints: Union[Unset, List[str]] = UNSET
        if not isinstance(self.fingerprints, Unset):
            fingerprints = self.fingerprints

        prefix = self.prefix
        disable_concurrent_reads = self.disable_concurrent_reads

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if endpoint is not UNSET:
            field_dict["endpoint"] = endpoint
        if username is not UNSET:
            field_dict["username"] = username
        if password is not UNSET:
            field_dict["password"] = password
        if private_key is not UNSET:
            field_dict["private_key"] = private_key
        if fingerprints is not UNSET:
            field_dict["fingerprints"] = fingerprints
        if prefix is not UNSET:
            field_dict["prefix"] = prefix
        if disable_concurrent_reads is not UNSET:
            field_dict["disable_concurrent_reads"] = disable_concurrent_reads

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        endpoint = d.pop("endpoint", UNSET)

        username = d.pop("username", UNSET)

        password: Union[Unset, Secret] = UNSET
        _password = d.pop("password", UNSET)
        if not isinstance(_password, Unset):
            password = Secret.from_dict(_password)

        private_key: Union[Unset, Secret] = UNSET
        _private_key = d.pop("private_key", UNSET)
        if not isinstance(_private_key, Unset):
            private_key = Secret.from_dict(_private_key)

        fingerprints = cast(List[str], d.pop("fingerprints", UNSET))

        prefix = d.pop("prefix", UNSET)

        disable_concurrent_reads = d.pop("disable_concurrent_reads", UNSET)

        sftp_fs_config = cls(
            endpoint=endpoint,
            username=username,
            password=password,
            private_key=private_key,
            fingerprints=fingerprints,
            prefix=prefix,
            disable_concurrent_reads=disable_concurrent_reads,
        )

        sftp_fs_config.additional_properties = d
        return sftp_fs_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.ftpd_binding_tls_mode import FTPDBindingTlsMode
from ..types import UNSET, Unset

T = TypeVar("T", bound="FTPDBinding")


@attr.s(auto_attribs=True)
class FTPDBinding:
    """  """

    address: Union[Unset, str] = UNSET
    port: Union[Unset, int] = UNSET
    apply_proxy_config: Union[Unset, bool] = UNSET
    tls_mode: Union[Unset, FTPDBindingTlsMode] = UNSET
    force_passive_ip: Union[Unset, str] = UNSET
    client_auth_type: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        address = self.address
        port = self.port
        apply_proxy_config = self.apply_proxy_config
        tls_mode: Union[Unset, int] = UNSET
        if not isinstance(self.tls_mode, Unset):
            tls_mode = self.tls_mode.value

        force_passive_ip = self.force_passive_ip
        client_auth_type = self.client_auth_type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if address is not UNSET:
            field_dict["address"] = address
        if port is not UNSET:
            field_dict["port"] = port
        if apply_proxy_config is not UNSET:
            field_dict["apply_proxy_config"] = apply_proxy_config
        if tls_mode is not UNSET:
            field_dict["tls_mode"] = tls_mode
        if force_passive_ip is not UNSET:
            field_dict["force_passive_ip"] = force_passive_ip
        if client_auth_type is not UNSET:
            field_dict["client_auth_type"] = client_auth_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        address = d.pop("address", UNSET)

        port = d.pop("port", UNSET)

        apply_proxy_config = d.pop("apply_proxy_config", UNSET)

        tls_mode: Union[Unset, FTPDBindingTlsMode] = UNSET
        _tls_mode = d.pop("tls_mode", UNSET)
        if not isinstance(_tls_mode, Unset):
            tls_mode = FTPDBindingTlsMode(_tls_mode)

        force_passive_ip = d.pop("force_passive_ip", UNSET)

        client_auth_type = d.pop("client_auth_type", UNSET)

        ftpd_binding = cls(
            address=address,
            port=port,
            apply_proxy_config=apply_proxy_config,
            tls_mode=tls_mode,
            force_passive_ip=force_passive_ip,
            client_auth_type=client_auth_type,
        )

        ftpd_binding.additional_properties = d
        return ftpd_binding

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.azure_blob_fs_config import AzureBlobFsConfig
from ..models.crypt_fs_config import CryptFsConfig
from ..models.filesystem_config_provider import FilesystemConfigProvider
from ..models.gcs_config import GCSConfig
from ..models.s3_config import S3Config
from ..models.sftp_fs_config import SFTPFsConfig
from ..types import UNSET, Unset

T = TypeVar("T", bound="FilesystemConfig")


@attr.s(auto_attribs=True)
class FilesystemConfig:
    """ Storage filesystem details """

    provider: Union[Unset, FilesystemConfigProvider] = UNSET
    s3config: Union[Unset, S3Config] = UNSET
    gcsconfig: Union[Unset, GCSConfig] = UNSET
    azblobconfig: Union[Unset, AzureBlobFsConfig] = UNSET
    cryptconfig: Union[Unset, CryptFsConfig] = UNSET
    sftpconfig: Union[Unset, SFTPFsConfig] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        provider: Union[Unset, int] = UNSET
        if not isinstance(self.provider, Unset):
            provider = self.provider.value

        s3config: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.s3config, Unset):
            s3config = self.s3config.to_dict()

        gcsconfig: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.gcsconfig, Unset):
            gcsconfig = self.gcsconfig.to_dict()

        azblobconfig: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.azblobconfig, Unset):
            azblobconfig = self.azblobconfig.to_dict()

        cryptconfig: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.cryptconfig, Unset):
            cryptconfig = self.cryptconfig.to_dict()

        sftpconfig: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.sftpconfig, Unset):
            sftpconfig = self.sftpconfig.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if provider is not UNSET:
            field_dict["provider"] = provider
        if s3config is not UNSET:
            field_dict["s3config"] = s3config
        if gcsconfig is not UNSET:
            field_dict["gcsconfig"] = gcsconfig
        if azblobconfig is not UNSET:
            field_dict["azblobconfig"] = azblobconfig
        if cryptconfig is not UNSET:
            field_dict["cryptconfig"] = cryptconfig
        if sftpconfig is not UNSET:
            field_dict["sftpconfig"] = sftpconfig

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        provider: Union[Unset, FilesystemConfigProvider] = UNSET
        _provider = d.pop("provider", UNSET)
        if not isinstance(_provider, Unset):
            provider = FilesystemConfigProvider(_provider)

        s3config: Union[Unset, S3Config] = UNSET
        _s3config = d.pop("s3config", UNSET)
        if not isinstance(_s3config, Unset):
            s3config = S3Config.from_dict(_s3config)

        gcsconfig: Union[Unset, GCSConfig] = UNSET
        _gcsconfig = d.pop("gcsconfig", UNSET)
        if not isinstance(_gcsconfig, Unset):
            gcsconfig = GCSConfig.from_dict(_gcsconfig)

        azblobconfig: Union[Unset, AzureBlobFsConfig] = UNSET
        _azblobconfig = d.pop("azblobconfig", UNSET)
        if not isinstance(_azblobconfig, Unset):
            azblobconfig = AzureBlobFsConfig.from_dict(_azblobconfig)

        cryptconfig: Union[Unset, CryptFsConfig] = UNSET
        _cryptconfig = d.pop("cryptconfig", UNSET)
        if not isinstance(_cryptconfig, Unset):
            cryptconfig = CryptFsConfig.from_dict(_cryptconfig)

        sftpconfig: Union[Unset, SFTPFsConfig] = UNSET
        _sftpconfig = d.pop("sftpconfig", UNSET)
        if not isinstance(_sftpconfig, Unset):
            sftpconfig = SFTPFsConfig.from_dict(_sftpconfig)

        filesystem_config = cls(
            provider=provider,
            s3config=s3config,
            gcsconfig=gcsconfig,
            azblobconfig=azblobconfig,
            cryptconfig=cryptconfig,
            sftpconfig=sftpconfig,
        )

        filesystem_config.additional_properties = d
        return filesystem_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.ftp_passive_port_range import FTPPassivePortRange
from ..models.ftpd_binding import FTPDBinding
from ..types import UNSET, Unset

T = TypeVar("T", bound="FTPServiceStatus")


@attr.s(auto_attribs=True)
class FTPServiceStatus:
    """  """

    is_active: Union[Unset, bool] = UNSET
    bindings: Union[Unset, None, List[FTPDBinding]] = UNSET
    passive_port_range: Union[Unset, FTPPassivePortRange] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        is_active = self.is_active
        bindings: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.bindings, Unset):
            if self.bindings is None:
                bindings = None
            else:
                bindings = []
                for bindings_item_data in self.bindings:
                    bindings_item = bindings_item_data.to_dict()

                    bindings.append(bindings_item)

        passive_port_range: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.passive_port_range, Unset):
            passive_port_range = self.passive_port_range.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if is_active is not UNSET:
            field_dict["is_active"] = is_active
        if bindings is not UNSET:
            field_dict["bindings"] = bindings
        if passive_port_range is not UNSET:
            field_dict["passive_port_range"] = passive_port_range

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        is_active = d.pop("is_active", UNSET)

        bindings = []
        _bindings = d.pop("bindings", UNSET)
        for bindings_item_data in _bindings or []:
            bindings_item = FTPDBinding.from_dict(bindings_item_data)

            bindings.append(bindings_item)

        passive_port_range: Union[Unset, FTPPassivePortRange] = UNSET
        _passive_port_range = d.pop("passive_port_range", UNSET)
        if not isinstance(_passive_port_range, Unset):
            passive_port_range = FTPPassivePortRange.from_dict(_passive_port_range)

        ftp_service_status = cls(
            is_active=is_active,
            bindings=bindings,
            passive_port_range=passive_port_range,
        )

        ftp_service_status.additional_properties = d
        return ftp_service_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.filesystem_config import FilesystemConfig
from ..types import UNSET, Unset

T = TypeVar("T", bound="BaseVirtualFolder")


@attr.s(auto_attribs=True)
class BaseVirtualFolder:
    """ Defines the filesystem for the virtual folder and the used quota limits. The same folder can be shared among multiple users and each user can have different quota limits or a different virtual path. """

    id: Union[Unset, int] = UNSET
    name: Union[Unset, str] = UNSET
    mapped_path: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    used_quota_size: Union[Unset, int] = UNSET
    used_quota_files: Union[Unset, int] = UNSET
    last_quota_update: Union[Unset, int] = UNSET
    users: Union[Unset, List[str]] = UNSET
    filesystem: Union[Unset, FilesystemConfig] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        mapped_path = self.mapped_path
        description = self.description
        used_quota_size = self.used_quota_size
        used_quota_files = self.used_quota_files
        last_quota_update = self.last_quota_update
        users: Union[Unset, List[str]] = UNSET
        if not isinstance(self.users, Unset):
            users = self.users

        filesystem: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.filesystem, Unset):
            filesystem = self.filesystem.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if mapped_path is not UNSET:
            field_dict["mapped_path"] = mapped_path
        if description is not UNSET:
            field_dict["description"] = description
        if used_quota_size is not UNSET:
            field_dict["used_quota_size"] = used_quota_size
        if used_quota_files is not UNSET:
            field_dict["used_quota_files"] = used_quota_files
        if last_quota_update is not UNSET:
            field_dict["last_quota_update"] = last_quota_update
        if users is not UNSET:
            field_dict["users"] = users
        if filesystem is not UNSET:
            field_dict["filesystem"] = filesystem

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        mapped_path = d.pop("mapped_path", UNSET)

        description = d.pop("description", UNSET)

        used_quota_size = d.pop("used_quota_size", UNSET)

        used_quota_files = d.pop("used_quota_files", UNSET)

        last_quota_update = d.pop("last_quota_update", UNSET)

        users = cast(List[str], d.pop("users", UNSET))

        filesystem: Union[Unset, FilesystemConfig] = UNSET
        _filesystem = d.pop("filesystem", UNSET)
        if not isinstance(_filesystem, Unset):
            filesystem = FilesystemConfig.from_dict(_filesystem)

        base_virtual_folder = cls(
            id=id,
            name=name,
            mapped_path=mapped_path,
            description=description,
            used_quota_size=used_quota_size,
            used_quota_files=used_quota_files,
            last_quota_update=last_quota_update,
            users=users,
            filesystem=filesystem,
        )

        base_virtual_folder.additional_properties = d
        return base_virtual_folder

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import functools
import importlib
import pkgutil
from datetime import datetime

import httpx

from sftpgo_client.base import api
from sftpgo_client.base.api.token.get_token import _build_response
from sftpgo_client.base.client import AuthenticatedClient
from sftpgo_client.base.models import *  # noqa


class Client(AuthenticatedClient):
    def __init__(self, base_url, user, password, *args, **kwargs):
        self.user = user
        self.password = password
        kwargs["token"] = ""
        super().__init__(base_url, *args, **kwargs)

    @property
    def token_expired(self):
        return self.token_expires_at >= datetime.utcnow()

    def _get_token__get_kwargs(self):
        return {
            "url": f"{self.base_url}/token",
            "headers": self.headers,
            "timeout": self.timeout,
            "auth": httpx.BasicAuth(username=self.user, password=self.password),
        }

    def _get_token__parse_response(self, response):
        response.raise_for_status()
        response = _build_response(response=response)
        return response.parsed

    def get_token(self):
        response = httpx.get(**self._get_token__get_kwargs())
        return self._get_token__parse_response(response)

    def _set_token(self, token):
        self.token = token.access_token
        self.token_expires_at = token.expires_at.replace(tzinfo=None)

    @classmethod
    def _make_proxy_method(cls, function):
        @functools.wraps(function)
        def proxy_method(self, *args, **kwargs):
            if not self.token or self.token_expired:
                token = self.get_token()
                self._set_token(token)

            kwargs["client"] = self
            return function(*args, **kwargs)

        return proxy_method

    @classmethod
    def _add_proxy_method(cls, method_name, function):
        proxy_method = cls._make_proxy_method(function)
        if not hasattr(cls, method_name):
            setattr(cls, method_name, proxy_method)


class AsyncClient(Client):
    async def get_token(self):
        async with httpx.AsyncClient() as _client:
            response = await _client.get(**self._get_token__get_kwargs())
        return self._get_token__parse_response(response)

    @classmethod
    def _make_proxy_method(cls, function):
        @functools.wraps(function)
        async def proxy_method(self, *args, **kwargs):
            if not self.token or self.token_expired:
                token = await self.get_token()
                self._set_token(token)

            kwargs["client"] = self
            return await function(*args, **kwargs)

        return proxy_method


for module_info in pkgutil.walk_packages(api.__path__):
    module_name = f"{api.__name__}.{module_info.name}"
    module = importlib.import_module(module_name)
    for submodule_info in pkgutil.walk_packages(module.__path__):
        submodule_name = f"{module_name}.{submodule_info.name}"
        submodule = importlib.import_module(submodule_name)
        if hasattr(submodule, "asyncio"):
            AsyncClient._add_proxy_method(submodule_info.name, submodule.asyncio)
        if hasattr(submodule, "sync"):
            Client._add_proxy_method(submodule_info.name, submodule.sync)



from enum import Enum


class AzureBlobFsConfigAccessTier(str, Enum):
    VALUE_0 = ""
    ARCHIVE = "Archive"
    HOT = "Hot"
    COOL = "Cool"

    def __str__(self) -> str:
        return str(self.value)



from setuptools import setup, find_packages


def get_description():
    with open("README.md") as file:
        return file.read()


setup(
    name="sftpgo-client",
    version="0.2.1",
    url="https://github.com/ramnes/sftpgo-client",
    author="Guillaume Gelin",
    author_email="contact@ramnes.eu",
    description="Python client for the SFTPGo API",
    long_description=get_description(),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    python_requires=">=3.6, <4",
    install_requires=[
        "httpx >= 0.15.0, < 0.18.0",
        "attrs >= 20.1.0",
        "python-dateutil >= 2.8.0, < 3",
    ],
)



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.get_users_order import GetUsersOrder
from ...models.user import User
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    offset: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 100,
    order: Union[Unset, GetUsersOrder] = UNSET,
) -> Dict[str, Any]:
    url = "{}/users".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_order: Union[Unset, str] = UNSET
    if not isinstance(order, Unset):
        json_order = order.value

    params: Dict[str, Any] = {
        "offset": offset,
        "limit": limit,
        "order": json_order,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[List[User], None, None, None, None]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = User.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == 400:
        response_400 = None

        return response_400
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 500:
        response_500 = None

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[List[User], None, None, None, None]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    offset: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 100,
    order: Union[Unset, GetUsersOrder] = UNSET,
) -> Response[Union[List[User], None, None, None, None]]:
    kwargs = _get_kwargs(
        client=client,
        offset=offset,
        limit=limit,
        order=order,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    offset: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 100,
    order: Union[Unset, GetUsersOrder] = UNSET,
) -> Optional[Union[List[User], None, None, None, None]]:
    """ Returns an array with one or more users. For security reasons hashed passwords are omitted in the response """

    return sync_detailed(
        client=client,
        offset=offset,
        limit=limit,
        order=order,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    offset: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 100,
    order: Union[Unset, GetUsersOrder] = UNSET,
) -> Response[Union[List[User], None, None, None, None]]:
    kwargs = _get_kwargs(
        client=client,
        offset=offset,
        limit=limit,
        order=order,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    offset: Union[Unset, int] = 0,
    limit: Union[Unset, int] = 100,
    order: Union[Unset, GetUsersOrder] = UNSET,
) -> Optional[Union[List[User], None, None, None, None]]:
    """ Returns an array with one or more users. For security reasons hashed passwords are omitted in the response """

    return (
        await asyncio_detailed(
            client=client,
            offset=offset,
            limit=limit,
            order=order,
        )
    ).parsed
